use std::{io::{Read, Write}, net::TcpListener};
use serde::{Serialize, Deserialize};
use serde_json;

const PORT: usize = 10000;
const BUFF_SIZE: usize = 1;

#[derive(Deserialize, Debug)]
struct Request {
    method: String,
    number: f64
}

#[derive(Serialize, Debug)]
struct Response {
    method: String,
    prime: bool
}

fn is_prime(n: f64) -> bool {
    if n <= 1.0 || n.floor() != n {
        return false;
    }

    let max_check = n.sqrt().floor() as usize;

    for i in 2..=max_check {
        let div = n / (i as f64);
        if div == div.floor() {
            return false;
        }
    }
    true
}

fn main() -> std::io::Result<()> {

    let listener = TcpListener::bind(format!("0.0.0.0:{PORT}"))?;

    for stream in listener.incoming() {
        std::thread::spawn(move || {
            match stream {
                Ok(mut stream) => {
                    let mut big_buf: Vec<u8> = vec![];
                    loop {
                        let mut buf: [u8; BUFF_SIZE] = [0; BUFF_SIZE];

                        if let Ok(_) = stream.read_exact(&mut buf) {
                            println!("{:?}", buf);
                            match buf {
                                [10] => {

                                    let s = String::from_utf8(big_buf.clone()).unwrap();
                                    big_buf.clear();

                                    println!("processing: {s}");

                                    let d = serde_json::from_str::<Request>(&s);

                                    match d {
                                        Ok(message) => {
                                            match message.method.as_str() {
                                                "isPrime" => {
                                                    let response = Response {
                                                            method: String::from("isPrime"),
                                                            prime: is_prime(message.number)
                                                        };
                                                    let res_str = format!("{}\n", serde_json::to_string(&response).unwrap());
                                                    let _ = stream.write(res_str.as_bytes());
                                                }
                                                s => {
                                                    println!("received invalid method: {s}");
                                                    let _ = stream.write(&"malformed".as_bytes());
                                                    let _ = stream.shutdown(std::net::Shutdown::Both);
                                                }
                                            }
                                        },
                                        Err(e) => {
                                            println!("received malformed request: {:?}", e);
                                            let _ = stream.write(&"malformed".as_bytes());
                                            let _ = stream.shutdown(std::net::Shutdown::Both);
                                        }
                                    }
                                },
                                [n] => {
                                    big_buf.push(n);
                                }
                            }
                        } else {
                            break;
                        }
                    }
                },
                Err(e) => panic!("{:?}", e)
            }
        });
    }

    Ok(())
}


#[cfg(test)]
mod tests {
    use super::is_prime;
    #[test]
    fn should_return_correct_primes() {
        assert!(!is_prime(8.0));
        assert!(is_prime(101.0));
        assert!(!is_prime(420.0));
        assert!(!is_prime(7.5));
         assert!(!is_prime(1.0));
        assert!(is_prime(76247027.0));
        assert!(is_prime(7.0));
    }
}